; Copyright (C) 2003  NASA/TRMM Satellite Validation Office
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;**************************************************************************
;
; rsl_wsr88d_to_radar
;
; Read a WSR-88D format file and return data in a Radar structure.
; 
; Syntax:
;     radar = rsl_wsr88d_to_radar(wsr88d_file [, siteid_or_header]
;                 [, /QUIET] [, ERROR=variable])
;
; Inputs:
;     wsr88d_file: name of WSR-88D file.
;     siteID_or_header:
;         a string containing either the 4-character WSR-88D site ID or
;         the name of a WSR-88D header file.  If omitted, program will
;         attempt to get site ID from file name.
;
; Keyword parameters:
;     QUIET: Set this keyword to turn off progress reporting.
;     ERROR: Assign a variable to this keyword to have a boolean error status
;         returned.  A value of 1 (true) indicates an error occurred, 0 means
;         no error.
;
; Written by:  Bart Kelley, GMU, January 2003
;
; Based on the RSL program RSL_wsr88d_to_radar by John Merritt of SM&A Corp.
;***********************************************************************

;***************************;
;     read_wsr88d_title     ;
;***************************;

pro read_wsr88d_title, iunit, title_rec

; Read title record and perform byte swapping where needed.

readu, iunit, title_rec
jdate = title_rec.jdate
msec = title_rec.msec
byteorder, jdate, /lswap, /swap_if_little_endian
byteorder, msec, /lswap, /swap_if_little_endian
title_rec.jdate = jdate
title_rec.msec = msec
end

;***************************;
;      read_wsr88d_rec      ;
;***************************;

pro read_wsr88d_rec, iunit, rec

; Read WSR-88D data record and perform integer byte swapping where needed.

readu,iunit,rec
longwords = lonarr(3)
longwords(0) = long(rec(10:11),0)
longwords(1) = long(rec(14:15),0)
longwords(2) = long(rec(30:31),0)
data = rec(64:1215) ; save the data.  don't want to byte-swap it.
byteorder, rec, /sswap, /swap_if_little_endian
byteorder, longwords, /lswap, /swap_if_little_endian
rec(10:11) = fix(longwords(0),0,2)
rec(14:15) = fix(longwords(1),0,2)
rec(30:31) = fix(longwords(2),0,2)
rec(64:1215) = data
end

;***************************;
;      get_vcp_params       ;
;***************************;

pro get_vcp_params, rec, azim_rate, fix_angle, pulse_width, pulse_count

; This procedure gets parameters dependent on Volume Coverage Pattern.

rsl_speed_of_light = 299792458.0
rda_elev_num = rec[22]
vcp = rec[36]

; The following values are from RSL's wsr88d.c, and were originally provided
; by Dan Austin, formerly of Texas A&M.
;
; The contents of these arrays are as follows:
; parms[0]   - VCP number
; parms[1]   - Pulse length
; parms[2-n] - triples consisting of fixed angle, pulse count, and azimuth rate,
;              for each Radar Data Acquisition (RDA) elevation (sweep).
;              Note that fixed angle and azimuth rate are encoded values.
;
; For example, for VCP 11 at RDA elevation 1:
;    fixed angle = parms[2] = 88
;    pulse count = parms[3] = 17
;    azimuth rate = parms[4] = 13600
; at RDA elevation 2:
;    fixed angle = parms[5] = 88
;    pulse count = parms[6] = 0
;    azimuth rate = parms[7] = 14000
;
; Code for unpacking these values follows CASE statement below.
;
; Note: two sweeps are made in each of the first two radar tilts in VCP 11 and
; VCP 21: the first for reflectivity, the second for velocity.  This means RDA
; elevation 2 is actually the second sweep of the first radar tilt.  For VCP 31
; and 32, two sweeps are made for the first *three* tilts.

case vcp of
    11: parms=[11,514, 88,17,13600,88,0,14000,264,16,12664,264,0,14000,$
               440,6,11736,608,6,24760,784,6,24760,952,10,12712,$
               1128,10,12720,1368,0,18328,1584,0,18496,1824,0,18512,$
               2184,0,18544,2552,0,18576,3040,0,18640,3552,0,18712]
    12: parms=[12, 514, 88,0,0,88,0,0,intarr(15*3)] 
    21: parms=[21,514, 88,28,8256,88,0,8272,264,28,8256,264,0,8272,$
               440,8,7888, 608,8,7888,784,8,8160,1096,12,8160,1800,0,10640,$
	       2656,0,10432, 3552,0,10496]
   121: parms=[121,514, 88,0,0,88,0,0,intarr(18*3)] 
    31: parms=[31,516, 88,63,3672,88,0,3688,272,63,3672,272,0,3688,$
               456,63,3672,456,0,3688,640,0,3688,816,0,3688]
    32: parms=[32,514, 88,64,3616,88,0,3312,272,64,3616,272,0,3312,$
               456,11,2960,456,0,2960,640,11,2960,816,11,2960]
endcase

; This code is also based on Dan Austin's in wsr88d.c.

fix_angle =   parms[(3*rda_elev_num)-1]/8.0*180./4096.0
pulse_count = parms[(3*rda_elev_num)]
azim_rate =   parms[(3*rda_elev_num)+1]/8.0*45./4096.0
pulse_width = parms[1]/(rsl_speed_of_light *1e-6)

end

;***************************;
;  load_radar_from_wsr88d   ;
;***************************;

pro load_radar_from_wsr88d, iunit, radar, quiet=quiet

rsl_speed_of_light = 299792458.0
datahdr_start = 28
no_data = radar.volume[0].h.no_data_flag

begin_elev   = 0
intrm_radial = 1
end_elev     = 2
beginvos     = 3
endvos       = 4

sweepindex = intarr(3) ; Current sweep index for each field.

; Read data records.

rec = intarr(1216)
endfile = 0
if not keyword_set(quiet) then quiet = 0
if not quiet then print, format='(/$,"Loading sweeps")'
while not endfile do begin
    read_wsr88d_rec, iunit, rec
    radstat = rec(20) ; Get radial status (start/end elevation, start/end vos)
    msgtype = rec(7) and 15
    if msgtype ne 1 then goto, nextrec
    if radstat eq beginvos or radstat eq begin_elev then begin ; new sweep
	sweepnum = rec[22]
	if not quiet then print, format='($,i4)', sweepnum
    endif
    jul2cal, rec[16], month, day, year
    sec = long(rec[14:15],0)/1000.
    hour = fix(sec/3600.)
    minute = fix((sec - hour*3600.)/60.)
    sec = sec - hour*3600. - minute*60.
    vcp = rec[36] ; Volume Coverage Pattern

    case vcp of
	11: sweeprate = 16./5.
	21: sweeprate = 11./6.
	31: sweeprate = 8./10.
	32: sweeprate = 8./10.
	12: sweeprate = 17./4.2
	121:sweeprate = 20./5.5
	else: sweeprate = 0.
    endcase

    ; Load values for individual fields into radar structure.

    for ivol = 0,2 do begin
	; if number of gates for this field is 0, skip to next field 
	if ivol gt 0 then ngates = rec[28] else ngates = rec[27]
	if ngates eq 0 then goto, nextfield
	isweep = sweepindex[ivol]
	iray = radar.volume[ivol].sweep[isweep].h.nrays
	radar.volume[ivol].sweep[isweep].ray[iray].h.month = month
	radar.volume[ivol].sweep[isweep].ray[iray].h.day = day
	radar.volume[ivol].sweep[isweep].ray[iray].h.year = year
	radar.volume[ivol].sweep[isweep].ray[iray].h.hour = hour
	radar.volume[ivol].sweep[isweep].ray[iray].h.minute = minute
	radar.volume[ivol].sweep[isweep].ray[iray].h.sec = sec
	radar.volume[ivol].sweep[isweep].ray[iray].h.ray_num = rec[19] 
	radar.volume[ivol].sweep[isweep].ray[iray].h.sweep_rate = sweeprate
	radar.volume[ivol].sweep[isweep].ray[iray].h.elev_num = sweepnum
	elev =(rec[21]/8.)*(180./4096.)
	radar.volume[ivol].sweep[isweep].ray[iray].h.elev = elev
	radar.volume[ivol].sweep[isweep].ray[iray].h.unam_rng = rec[17]/10.
	if ivol gt 0 then rangeptr = 24 else rangeptr = 23
	radar.volume[ivol].sweep[isweep].ray[iray].h.range_bin1 = rec[rangeptr]
	radar.volume[ivol].sweep[isweep].ray[iray].h.gate_size = rec[rangeptr+2]
	azimuth = (rec[18]/8.)*(180./4096.)
	if azimuth lt 0. then azimuth = azimuth + 360. 
	radar.volume[ivol].sweep[isweep].ray[iray].h.azimuth = azimuth
	radar.volume[ivol].sweep[isweep].ray[iray].h.beam_width = .95
	radar.volume[ivol].sweep[isweep].ray[iray].h.nbins = ngates
	prf = rsl_speed_of_light /(2.*rec[17]*100.)
	radar.volume[ivol].sweep[isweep].ray[iray].h.prf = round(prf)
	nyq_vel = rec[44]/100.
	radar.volume[ivol].sweep[isweep].ray[iray].h.nyq_vel = nyq_vel 
	; code for wavelength and frequency is from RSL's wsr88d.c
	if prf eq 0. or nyq_vel eq 0. then wavelength = .1 $
	else wavelength = 4.* nyq_vel / prf
	radar.volume[ivol].sweep[isweep].ray[iray].h.wavelength = wavelength
	radar.volume[ivol].sweep[isweep].ray[iray].h.frequency = $
	    rsl_speed_of_light / wavelength * 1e-9
	get_vcp_params, rec, azim_rate, fix_angle, pulse_width, pulse_count
	radar.volume[ivol].sweep[isweep].ray[iray].h.azim_rate = azim_rate 
	radar.volume[ivol].sweep[isweep].ray[iray].h.fix_angle = fix_angle
	radar.volume[ivol].sweep[isweep].ray[iray].h.pulse_count = pulse_count 
	radar.volume[ivol].sweep[isweep].ray[iray].h.pulse_width = pulse_width 
	; get the data.
	velrescode = rec[35]
	dataptr = rec[ivol+32]
	data = byte(rec,datahdr_start+dataptr,ngates)
	s = where(data eq 0 or data eq 1)
	range  =  fltarr(ngates)
	if size(s,/n_dimensions) gt 0 then range[s] = no_data
	s = where(data gt 1)
	if size(s,/n_dimensions) gt 0 then $
	    if ivol eq 0 then range[s] = (float(data[s])-2.)/2.-32. $
	    else if ivol eq 2 then range[s] = (float(data[s])-2.)/2.-63.5 $
	    else if velrescode eq 2 then range[s] = (float(data[s])-2.)/2.-63.5 $
	    else range[s] = (float(data[s])-2.)/2.-127.
	radar.volume[ivol].sweep[isweep].ray[iray].range[0:ngates-1] = range
	radar.volume[ivol].sweep[isweep].h.nrays = iray + 1
	case velrescode of
	    2: velres = 0.5
	    4: velres = 1.0
	else: velres = 0.
	endcase
	radar.volume[ivol].sweep[isweep].ray[iray].h.vel_res = velres
    nextfield:
    endfor
    if radstat eq end_elev or radstat eq endvos then begin
	load_sweep_info_wsr88d, sweepindex, sweepnum, radar
	for ivol = 0,n_elements(sweepindex)-1 do $
	    if radar.volume[ivol].sweep[sweepindex[ivol]].h.nrays ne 0 then $
	        sweepindex[ivol] = sweepindex[ivol] + 1
    endif
nextrec: if radstat eq endvos or eof(iunit) then endfile = 1
endwhile ; while not endvos

if not quiet then print ; Print newline at end of sweep numbers list.
end

;****************************;
;  load_headers_from_wsr88d  ;
;****************************;

pro load_headers_from_wsr88d, radar, siteinfo

radar.volume.h.field_type = ['DZ','VR','SW']
radar.volume[0].sweep.h.field_type = 'DZ'
radar.volume[1].sweep.h.field_type = 'VR'
radar.volume[2].sweep.h.field_type = 'SW'

radar.h.number = siteinfo.sitenum
radar.h.name =   siteinfo.siteid
radar.h.radar_name = siteinfo.siteid
radar.h.city =   siteinfo.city
radar.h.state =  siteinfo.state
; If lat/lon degrees are negative, so are minutes and seconds.
if siteinfo.latd lt 0 then begin
    siteinfo.latm = -1 * abs(siteinfo.latm)
    siteinfo.lats = -1 * abs(siteinfo.lats)
endif
if siteinfo.lond lt 0 then begin
    siteinfo.lonm = -1 * abs(siteinfo.lonm)
    siteinfo.lons = -1 * abs(siteinfo.lons)
endif
radar.h.latd =   siteinfo.latd
radar.h.latm =   siteinfo.latm
radar.h.lats =   siteinfo.lats
radar.h.lond =   siteinfo.lond
radar.h.lonm =   siteinfo.lonm
radar.h.lons =   siteinfo.lons
radar.h.height = siteinfo.height
; Short and long pulse values are from RSL wsr88d_get_site.c.
radar.h.spulse = 1530
radar.h.lpulse = 4630 
radar.h.month =  radar.volume[0].sweep[0].ray[0].h.month
radar.h.day =    radar.volume[0].sweep[0].ray[0].h.day
radar.h.year =   radar.volume[0].sweep[0].ray[0].h.year
radar.h.hour =   radar.volume[0].sweep[0].ray[0].h.hour
radar.h.minute = radar.volume[0].sweep[0].ray[0].h.minute
radar.h.sec =    radar.volume[0].sweep[0].ray[0].h.sec
radar.h.radar_type = 'wsr88d'
end

;***************************;
;  load_sweep_info_wsr88d   ;
;***************************;

pro load_sweep_info_wsr88d, sweepindex, sweepnum, radar

for ivol = 0, radar.h.nvolumes-1 do begin
    isweep = sweepindex[ivol]
    nrays = radar.volume[ivol].sweep[isweep].h.nrays
    if nrays gt 0 then begin
	nsweeps = radar.volume[ivol].h.nsweeps
	radar.volume[ivol].h.nsweeps = nsweeps + 1
	radar.volume[ivol].sweep[isweep].h.sweep_num = sweepnum
	s = where(radar.volume[ivol].sweep[isweep].ray.h.azimuth ne -99999.)
	;; NOTE: maybe use fixed angle instead of average angle.
	elev = total(radar.volume[ivol].sweep[isweep].ray[s].h.elev)/nrays
	radar.volume[ivol].sweep[isweep].h.elev = elev
    endif
endfor
beamwidth = radar.volume[0].sweep[isweep].ray[0].h.beam_width
radar.volume.sweep[isweep].h.beam_width = beamwidth
radar.volume.sweep[isweep].h.vert_half_bw = beamwidth  / 2.
radar.volume.sweep[isweep].h.horz_half_bw = beamwidth  / 2.
end

;***************************;
;    rsl_wsr88d_to_radar    ;
;***************************;

function rsl_wsr88d_to_radar, wsr88d_file, siteid_or_header, quiet=quiet, $
    error=error

; Read a WSR-88D format file and return data in a Radar structure.

maxsweeps = 20
maxrays = 400
maxbins = 1000
nfields = 3

radar = -1
error = 0

; Get information for the radar header based on site ID.  siteid argument can be
; true 4 character site id or name of a WSR-88D header file.  if header file is
; given, site id will be read from it.  if siteid is omitted, program will
; attempt to get it from wsr88d file name.

; I store filename in a string array and use the first element of array for
; passing it to other routines.  This way filename string is passed by value.

filename = [wsr88d_file, '']

if n_elements(siteid_or_header) eq 0 then sitename = filename[0] $
else if strlen(siteid_or_header) gt 4 then begin
    wsr88d_header = wsr88d_read_tape_header(siteid_or_header)
    sitename = wsr88d_header.siteid
endif else sitename = siteid_or_header

siteinfo = wsr88d_get_site_info(sitename)

; Open wsr88d file, uncompressing if necessary.

if is_compressed(filename[0]) then begin
    tmp_file = rsl_uncompress(filename[0])
    ; next line is a safeguard--we wouldn't want to delete a data file.
    if tmp_file eq wsr88d_file then message, 'Temporary file name returned by'+$
        ' rsl_uncompress is same as original: ' + tmp_file
    openr, iunit, tmp_file, /get_lun, /delete ; this file is deleted on close.
endif else openr, iunit, wsr88d_file, /get_lun

; Read first record of file (header record).

title_rec = {archivetitle:bytarr(12),jdate:0L,msec:0L,unused:0L}
read_wsr88d_title, iunit, title_rec
if string(title_rec.archivetitle[0:8]) ne 'ARCHIVE2.' and $
   string(title_rec.archivetitle[0:8]) ne 'AR2V0001.' then begin
        message, wsr88d_file + ' is not a NEXRAD file.', /continue
	goto, finished
endif

jul2cal, 0 ; Placed here to get compiler message out of the way.
radar = rsl_new_radar(nfields, maxsweeps, maxrays, maxbins)
radar.h.nvolumes = nfields
radar.volume.h.no_data_flag = -32767.
load_radar_from_wsr88d, iunit, radar, quiet=quiet
load_headers_from_wsr88d, radar, siteinfo

finished:
free_lun, iunit
if size(radar,/n_dimensions) eq 0 then error = 1
return, radar
end
